/*Includes ----------------------------------------------*/
#include "tool.h"
#include "MahonyAHRS.h"
#include "ble_comm.h"
#include "nrf_delay.h"
#include "bsp_pwm.h"
#include "bsp_time.h"
#include "fml_adc.h"
#include "bsp_wdt.h"
#include "exception.h"
#include "selfcheck.h"
#include "system.h"
#include "drv_qma7981.h"
#include "hal_led.h"
#include "hal_battery.h"
#include "bll_imu.h"
#include "app_flash.h"
#include "system.h"

/*Private macro ------------------------------------------------------------------------------------------------------------------------------------*/

#define	SELFCHECK_SCAN_DEVICE_RSSI_MIN_THRESHOLD					                     -60																								//Ҫɨ豸RSSIСֵ-76

#define	SELFCHECK_CHARGE_CHIP_PIN_ADC_MIN_THRESHOLD				                     80																									//оƬŵСADCֵ

#define	SELFCHECK_BATTERY_PIN_ADC_MIN_THRESHOLD						                     3300																								//طѹСADCֵ

#define	SELFCHECK_BATTERY_PIN_ADC_MAX_THRESHOLD						                     4300																								//طѹADCֵԭʼֵΪ3721Ϊ5.49V

#define SELFCHECK_MIDDLE_ACC_PROS_ROLL_MIN_THRESHOLD													 0																									//мٶROLLֵСֵ

#define	SELFCHECK_MIDDLE_ACC_PROS_ROLL_MAX_THRESHOLD													 35																									//мٶROLLֵֵ

#define SELFCHECK_MIDDLE_ACC_CONS_ROLL_MIN_THRESHOLD													 160																								//мٶȷROLLֵСֵ

#define	SELFCHECK_MIDDLE_ACC_CONS_ROLL_MAX_THRESHOLD													 180																								//мٶȷROLLֵֵ

#define SELFCHECK_WEAR_INSOLE_MAG_NORM_MIN_THRESHOLD													 5000																								//ЬĵشnormֵСֵ

#define	SELFCHECK_SENSOR_MAG_SHAKE_TRIGGER_THRESHOLD													 18000																							//شŶ

#define SELFCHECK_SENSOR_MAG_NO_WELDING_CAPACITOR_MIN_THRESHOLD								 200																								//شŴûӵݵСֵ

#define	SELFCHECK_SENSOR_MAG_SHAKE_THRESHOLD																	 200																								//شŶֵ


/*STRUCTION ------------------------------------------------------------------------------------------------------------------------------------*/
typedef enum {
	SELFCHECK_RESULT_SUCCESS = 0,																																																		//Լɹ
	
	SELFCHECK_RESULT_ERR_FRONT_SENSOR_CONFIG_SIX_AXIS,																																							//ԼʧܡǰŴ
	
	SELFCHECK_RESULT_ERR_FRONT_SENSOR_CONFIG_MAG,																																										//ԼʧܡǰŴõش
	
	SELFCHECK_RESULT_ERR_FRONT_SIX_AXIS_NO_DATA_OR_EXCP_DATA,																																				//ԼʧܡǰŴûݻ쳣
	
	SELFCHECK_RESULT_ERR_FRONT_MAG_NO_DATA_OR_EXCP_DATA,																																						//ԼʧܡǰŴشûݻ쳣
	
	SELFCHECK_RESULT_ERR_BACK_SENSOR_CONFIG,																																												//ԼʧܡŴ
	
	SELFCHECK_RESULT_ERR_BACK_NO_DATA_OR_EXCP_DATA,																																									//ԼʧܡŴûݻ쳣
	
	SELFCHECK_RESULT_ERR_MIDDLE_SENSOR_CONFIG,																																											//Լʧܡм䴫
	
	SELFCHECK_RESULT_ERR_MIDDLE_NO_DATA_OR_EXCP_DATA,																																								//Լʧܡм䴫ûݻ쳣
	
	SELFCHECK_RESULT_ERR_CHARGE_CHIP_PIN_ADC,																																												//ԼʧܡоƬADC
	
	SELFCHECK_RESULT_ERR_BATTERY_PIN_ADC,																																														//ԼʧܡطѹADC
	
	SELFCHECK_RESULT_ERR_RSSI,																																																			//ԼʧܡRSSI
	
} SELFCHECK_RESULT_e;

typedef struct _selfcheck
{
	uint32_t																  			  selfcheck_result;																															//Լ
	
	uint32_t     																			selfcheck_result_led_color;																										//Լledɫ
	
	uint32_t																					selfcheck_result_flash_num;																										//Լ˸
	
	bool																							selfcheck_is_led_display;																											//ԼledǷʾ
	
	int16_t 																					max_rssi;																																			//RSSIֵ
	
	fml_imu_data_t 																		f_data;																																				//ǰŴ
	
	fml_imu_data_t 																		b_data;																																				//Ŵ
	
	qma_data_t 																				m_data;																																				//м䴫
	
	bool																							f_is_read_data;																																//ǰŴǷݱ־λ
	
	bool																						  b_is_read_data;																																//ŴǷݱ־λ
	
} SelfCheck_t;

/*Local Variable ------------------------------------------------------------------------------------------------------------------------------------*/
static SelfCheck_t ob_selfcheck;

static const bll_imu_one_way_param_t game_front_param={
	.acc_power_mode 						= FML_IMU_ACC_POWER_MODE_NORMAL,						//ǰ - ٶģʽ
	.gry_power_mode 						= FML_IMU_GRY_POWER_MODE_NORMAL,						//ǰ - ģʽ
	.timestamp_resolution 			= FML_IMU_TIMESTAMP_25US,										//ǰ - ʱ25US
	.timestamp_switch 					= FML_IMU_TIMESTAMP_ON,											//ǰ - ʱ
	.acc_fs 										= FML_IMU_ACC_FS_16G,												//ǰ - ٶ - 16G
	.gry_fs 										= FML_IMU_GRY_FS_2000DPS,										//ǰ -  - 2000DPS
	.mag_fs 										= FML_IMU_MAG_FS_30GS,											//ǰ - شż - 30GS
	.acc_odr 									  = FML_IMU_ACC_ODR_416HZ,										//ǰ - ٶȲƵ - 104HZ
	.gry_odr 									  = FML_IMU_GRY_ODR_416HZ,										//ǰ - ǲƵ - 104HZ
	.mag_odr 									  = FML_IMU_MAG_ODR_200HZ,										//ǰ - شżƲƵ - 200HZ
	.fifo_odr 									= FML_IMU_FIFO_ODR_416HZ,	
};

static const bll_imu_one_way_param_t game_back_param={
	.acc_power_mode 						= FML_IMU_ACC_POWER_MODE_NORMAL,						// - ٶģʽ
	.gry_power_mode 						= FML_IMU_GRY_POWER_MODE_NORMAL,						// - ģʽ
	.timestamp_resolution 			= FML_IMU_TIMESTAMP_25US,										// - ʱ25US
	.timestamp_switch 					= FML_IMU_TIMESTAMP_OFF,										// - ʱر
	.acc_fs 										= FML_IMU_ACC_FS_16G,											  // - ٶ - 16G
	.gry_fs 										= FML_IMU_GRY_FS_2000DPS,										// -  - 2000DPS
	.mag_fs 										= FML_IMU_MAG_FS_30GS,											// - شż - 30GS
	.acc_odr 										= FML_IMU_ACC_ODR_OFF,											// - ٶȲƵ - ر
	.gry_odr 										= FML_IMU_GRY_ODR_OFF,											// - ǲƵ - ر
	.mag_odr 										= FML_IMU_MAG_ODR_200HZ,										// - شżƲƵ - 200HZ
	.fifo_odr 									= FML_IMU_FIFO_ODR_OFF,	
};

static const bll_imu_param_t game_bll_imu_param_t={
	.config_param[FML_IMU_DIR_FRONT] = &game_front_param,
	.config_param[FML_IMU_DIR_BACK] =  &game_back_param,
};

/*Local Functions ------------------------------------------------------------------------------------------------------------------------------------*/
static void fb_data_notify_cb(uint32_t dir_bit)
{
	int data_len;

	if((dir_bit >> BLL_IMU_DIR_FRONT) & 0x01)
	{
		memset(&ob_selfcheck.f_data,0,sizeof(ob_selfcheck.f_data));
		data_len = bll_imu_get_data_num(BLL_IMU_DIR_FRONT);
		for(int i=0; i<data_len;i++)
		{
			bll_imu_get_data(BLL_IMU_DIR_FRONT, i, &ob_selfcheck.f_data);
		}
	}
	
	if((dir_bit >> BLL_IMU_DIR_BACK) & 0x01)
	{
		memset(&ob_selfcheck.b_data,0,sizeof(ob_selfcheck.b_data));
		data_len = bll_imu_get_data_num(BLL_IMU_DIR_BACK);
		for(int i=0; i<data_len;i++)
		{
			bll_imu_get_data(BLL_IMU_DIR_BACK, i, &ob_selfcheck.b_data);
		}
	}
	
	if(((dir_bit >> BLL_IMU_DIR_FRONT) & 0x01))
	{
		 ob_selfcheck.f_is_read_data = true;
	}

	
	if((dir_bit >> BLL_IMU_DIR_BACK))
	{
		ob_selfcheck.b_is_read_data = true;
	}
	
}

static void scan_report_cb(uint8_t *adv_data, uint16_t adv_data_len,int8_t rssi)
{
	ob_selfcheck.max_rssi = (ob_selfcheck.max_rssi > rssi)?ob_selfcheck.max_rssi:rssi;
}

static void selfcheck_led_display_process(void)
{
	static uint32_t cur_flash_num = 0;
	static uint8_t	level = LED_ENABLE;
	static uint8_t	red_off_led_count = 0;
	static uint32_t tim = 0;
	
	//ι
	feed_watchdog();

	//led display
	if(ob_selfcheck.selfcheck_is_led_display == true)
	{
		if(cur_flash_num == 0)tim = TIME_GetTicks();
		
		//жǷ
		if(level == LED_ENABLE)cur_flash_num++;
		if(cur_flash_num > ob_selfcheck.selfcheck_result_flash_num)
		{
			nrf_gpio_pin_write(PIN_LED_ENABLE,LED_DISABLE);
			if(ob_selfcheck.selfcheck_result_led_color == COLOR_RED && (TIME_GetTicks() - tim < 5000))return;
			cur_flash_num = 0;
			level = LED_ENABLE;
			red_off_led_count = 0;
			ob_selfcheck.selfcheck_is_led_display = false;
			Process_Stop(selfcheck_led_display_process);
			return;
		}
		
		
		if(ob_selfcheck.selfcheck_result_led_color == COLOR_RED)
		{
			Process_Start(50,"selfcheck_led_display_process",selfcheck_led_display_process);
			nrf_gpio_pin_write(PIN_LED_ENABLE,level);nrf_delay_ms(5);
			if(level == LED_ENABLE){WS2812_DisplayDot(ob_selfcheck.selfcheck_result_led_color);WS2812_Pwm_Play();}
		}
		else
		{
			Process_Start(500,"selfcheck_led_display_process",selfcheck_led_display_process);
			nrf_gpio_pin_write(PIN_LED_ENABLE,level);nrf_delay_ms(5);
			if(level == LED_ENABLE){WS2812_DisplayDot(ob_selfcheck.selfcheck_result_led_color);WS2812_Pwm_Play();}
		}
		
		//ڷת
		if(ob_selfcheck.selfcheck_result_led_color == COLOR_RED)
		{
			if(level == LED_ENABLE){level = LED_DISABLE;red_off_led_count = 0;}
			if(level == LED_DISABLE){red_off_led_count++;if(red_off_led_count >= 6)level = LED_ENABLE;}
		}
		else
		{
			level = (level == LED_ENABLE)?LED_DISABLE:LED_ENABLE;
		}
	}
}

static void selfcheck_result_display_process(void)
{
	uint16_t 				front_mag_norm;
	
	//ι
	feed_watchdog();
	
	if(ob_selfcheck.selfcheck_is_led_display == false)
	{
		//Լʾ
		//ǰŴɫǰ˸1£ǰŵش˸2£ǰݶȡʧܻ쳣˸3£ǰŵشݶȡʧܻ쳣˸4£
		//Ŵɫŵش˸5£ŵشݶȡʧܻ쳣˸6£
		//м䴫ɫмٶ˸7£ٶrollֵڷΧ˸8£
		//оƬ͵طѹrssiɫоƬ˸9£طѹ˸10£rssi˸11£
		//мٶȼ𶯵
		//ͨɫ1ڣ500ms500ms𣨶ϵԴߣ⵽Ь棬ɫ1ڣ500ms500ms𣨶ϵԴߣ 
		//LEDԴߣ͡4ڣ40ms160ms𣨶ϵԴߣһ飩
		
		//ȡɫ+˸+
		if(ob_selfcheck.selfcheck_result == 0)
		{
			front_mag_norm = sqrt((double)(ob_selfcheck.f_data.mag[0]*ob_selfcheck.f_data.mag[0]) + (double)(ob_selfcheck.f_data.mag[1]*ob_selfcheck.f_data.mag[1]) + (double)(ob_selfcheck.f_data.mag[2]*ob_selfcheck.f_data.mag[2]));	
			if(SELFCHECK_WEAR_INSOLE_MAG_NORM_MIN_THRESHOLD <= front_mag_norm)
			{
				ob_selfcheck.selfcheck_result_led_color = COLOR_GREEN;
				ob_selfcheck.selfcheck_result_flash_num = 1;
			}
			else
			{
				ob_selfcheck.selfcheck_result_led_color = COLOR_BLUE;
				ob_selfcheck.selfcheck_result_flash_num = 1;
			}	
		}
		else
		{
			ob_selfcheck.selfcheck_result_led_color = COLOR_RED;
			for(int i=0;i<sizeof(ob_selfcheck.selfcheck_result)*8;i++)
			{
				if((ob_selfcheck.selfcheck_result & (1 << i)) != 0)
				{
					ob_selfcheck.selfcheck_result_flash_num = i;
					break;
				}
			}	
		}
		Process_Start(0,"selfcheck_led_display_process",selfcheck_led_display_process);
		ob_selfcheck.selfcheck_is_led_display = true;
	}
}

static void selfcheck_mt_process(void)
{
	//ι
	feed_watchdog();	
	
	nrf_gpio_pin_toggle(PIN_MT_EN);
}

static void selfcheck_process(void)
{
	BLL_IMU_CONFIG_RESULT  	f_config_result,b_config_result;
	int16_t									adc_value = 0;
	uint16_t 								front_mag_norm;
	uint16_t 								back_mag_norm;
	int16_t 								roll;
	uint8_t 								buf[256];
	uint8_t 								L=0;
	char 										mac_buf[16];	
	static uint32_t					charge_chip_adc_max = 0;
	static int16_t					battery_adc_max = 0;
	static uint32_t 				t_count = 0;
	static uint32_t					battery_adc_t_count = 0;
	static uint32_t					roll_t_count = 0;
	static uint32_t         sensor_t_count = 0;
	static uint32_t					last_tim = 0;
	static uint32_t					continue_trigger = 0;
	static uint32_t					front_mag_norm_max = 0;
	static uint32_t					back_mag_norm_max = 0;
	static uint32_t					front_mag_norm_min = 0xffffffff;
	static uint32_t					back_mag_norm_min = 0xffffffff;
	static bool							front_mag_shake_trigger = false;
	static bool							back_mag_shake_trigger = false;

	t_count++;
	
	//ι
	feed_watchdog();
	
	//ȡм䴫ݣٶrollֵ
	memset(&ob_selfcheck.m_data,0,sizeof(ob_selfcheck.m_data));
	if(drv_qma_get_acc_data(&ob_selfcheck.m_data) != 0)
	{
		ob_selfcheck.selfcheck_result |= (1 << SELFCHECK_RESULT_ERR_MIDDLE_NO_DATA_OR_EXCP_DATA);
		//
		drv_qma_set_acc_odr(QMA_ACC_ODR_104HZ);		
	}
	Mahony_process(0,0,0,ob_selfcheck.m_data.acc[0],ob_selfcheck.m_data.acc[1],ob_selfcheck.m_data.acc[2],0,0,0);
	
	
	//ɸѡĵطѹĵѹ
	fml_adc_get_value(PIN_ADC_BAT_CHANNEL,&adc_value);
	adc_value = ADC_RESULT_IN_MILLI_VOLTS(adc_value) * 5 / 3;
	battery_adc_max = adc_value;
	
	//ÿ3ȡADC5VѹԵطѹǷ񺸽ӣֵԼ
	if(t_count - battery_adc_t_count > (3000/100))
	{
		if(!(SELFCHECK_BATTERY_PIN_ADC_MIN_THRESHOLD < battery_adc_max && battery_adc_max < SELFCHECK_BATTERY_PIN_ADC_MAX_THRESHOLD))ob_selfcheck.selfcheck_result |= (1 << SELFCHECK_RESULT_ERR_BATTERY_PIN_ADC);
		else ob_selfcheck.selfcheck_result &= ~(1 << SELFCHECK_RESULT_ERR_BATTERY_PIN_ADC);
		//¼ʱ
		battery_adc_t_count = t_count;
	}
	
	//оƬ
	fml_adc_get_value(PIN_ADC_CHARGMEASURE_CHANNEL,&adc_value);
	if(SELFCHECK_CHARGE_CHIP_PIN_ADC_MIN_THRESHOLD < adc_value)
	{
		charge_chip_adc_max++;
	}
	if(charge_chip_adc_max > 10)
	{
		ob_selfcheck.selfcheck_result &= ~(1 << SELFCHECK_RESULT_ERR_CHARGE_CHIP_PIN_ADC);
	}
	else
	{
		ob_selfcheck.selfcheck_result |= (1 << SELFCHECK_RESULT_ERR_CHARGE_CHIP_PIN_ADC);
	}
	
	

	//ÿ3ٶrollֵԼ
	if(t_count - roll_t_count > (3000/100))
	{
		roll  = (int16_t)(getRoll()*100);
		if(roll < 0)roll *= -1;
		roll = (roll > 0 && roll < 100)?1:roll/100;
		if(!( \
		(SELFCHECK_MIDDLE_ACC_PROS_ROLL_MIN_THRESHOLD < roll && roll < SELFCHECK_MIDDLE_ACC_PROS_ROLL_MAX_THRESHOLD) || \
		(SELFCHECK_MIDDLE_ACC_CONS_ROLL_MIN_THRESHOLD < roll && roll < SELFCHECK_MIDDLE_ACC_CONS_ROLL_MAX_THRESHOLD) \
		))ob_selfcheck.selfcheck_result |= (1 << SELFCHECK_RESULT_ERR_MIDDLE_NO_DATA_OR_EXCP_DATA);
		else
			ob_selfcheck.selfcheck_result &= ~(1 << SELFCHECK_RESULT_ERR_MIDDLE_NO_DATA_OR_EXCP_DATA);
		//¼ʱ
		roll_t_count = t_count;
	}

	//ÿ1鿴ǰ󴫸ǷҶȡɹҪ1нֵǷشûкӵݣԼ
	if(t_count - sensor_t_count > (1000/100))
	{
		//ǰŴж
		f_config_result = bll_imu_query_config_param_is_ready(BLL_IMU_DIR_FRONT,&game_bll_imu_param_t);
		if(f_config_result != BLL_IMU_CONFIG_FINISH)
		{
			if(f_config_result == BLL_IMU_CONFIG_FAIL_FRONT_MAG)ob_selfcheck.selfcheck_result |= (1 << SELFCHECK_RESULT_ERR_FRONT_SENSOR_CONFIG_MAG);
			if(f_config_result == BLL_IMU_CONFIG_FAIL_FRONT_SIX_AXIS)ob_selfcheck.selfcheck_result |= (1 << SELFCHECK_RESULT_ERR_FRONT_SENSOR_CONFIG_SIX_AXIS);
		}
		else
		{
			//û
			ob_selfcheck.selfcheck_result &= ~(1 << SELFCHECK_RESULT_ERR_FRONT_SENSOR_CONFIG_MAG);
			ob_selfcheck.selfcheck_result &= ~(1 << SELFCHECK_RESULT_ERR_FRONT_SENSOR_CONFIG_SIX_AXIS);
			
			if(ob_selfcheck.f_is_read_data != true)ob_selfcheck.selfcheck_result |= (1 << SELFCHECK_RESULT_ERR_FRONT_SIX_AXIS_NO_DATA_OR_EXCP_DATA);
			else
			{
				//û
				ob_selfcheck.selfcheck_result &= ~(1 << SELFCHECK_RESULT_ERR_FRONT_SIX_AXIS_NO_DATA_OR_EXCP_DATA);
				ob_selfcheck.f_is_read_data = false;
				
				front_mag_norm = sqrt((double)(ob_selfcheck.f_data.mag[0]*ob_selfcheck.f_data.mag[0]) + (double)(ob_selfcheck.f_data.mag[1]*ob_selfcheck.f_data.mag[1]) + (double)(ob_selfcheck.f_data.mag[2]*ob_selfcheck.f_data.mag[2]));
					
				//ǰŴشûӵ
				if(front_mag_norm < SELFCHECK_SENSOR_MAG_NO_WELDING_CAPACITOR_MIN_THRESHOLD)ob_selfcheck.selfcheck_result |= (1 << SELFCHECK_RESULT_ERR_FRONT_MAG_NO_DATA_OR_EXCP_DATA);
				else ob_selfcheck.selfcheck_result &= ~(1 << SELFCHECK_RESULT_ERR_FRONT_MAG_NO_DATA_OR_EXCP_DATA);
			}
		}
		
		
		//Ŵж
		b_config_result = bll_imu_query_config_param_is_ready(BLL_IMU_DIR_BACK,&game_bll_imu_param_t);
		if(b_config_result != BLL_IMU_CONFIG_FINISH)ob_selfcheck.selfcheck_result |= (1 << SELFCHECK_RESULT_ERR_BACK_SENSOR_CONFIG);
		else 
		{
			//û
			ob_selfcheck.selfcheck_result &= ~(1 << SELFCHECK_RESULT_ERR_BACK_SENSOR_CONFIG);
			
			if(ob_selfcheck.b_is_read_data != true)ob_selfcheck.selfcheck_result |= (1 << SELFCHECK_RESULT_ERR_BACK_NO_DATA_OR_EXCP_DATA);
			else
			{
				//û
				ob_selfcheck.selfcheck_result &= ~(1 << SELFCHECK_RESULT_ERR_BACK_NO_DATA_OR_EXCP_DATA);
				ob_selfcheck.b_is_read_data = false;
				
				back_mag_norm = sqrt((double)(ob_selfcheck.b_data.mag[0]*ob_selfcheck.b_data.mag[0]) + (double)(ob_selfcheck.b_data.mag[1]*ob_selfcheck.b_data.mag[1]) + (double)(ob_selfcheck.b_data.mag[2]*ob_selfcheck.b_data.mag[2]));
					
				//Ŵشûӵ
				if(back_mag_norm < SELFCHECK_SENSOR_MAG_NO_WELDING_CAPACITOR_MIN_THRESHOLD)ob_selfcheck.selfcheck_result |= (1 << SELFCHECK_RESULT_ERR_BACK_NO_DATA_OR_EXCP_DATA);
				else ob_selfcheck.selfcheck_result &= ~(1 << SELFCHECK_RESULT_ERR_BACK_NO_DATA_OR_EXCP_DATA);
			}
			
		}

		//⴫ʧܣ
		if(f_config_result != BLL_IMU_CONFIG_FINISH || b_config_result != BLL_IMU_CONFIG_FINISH || ob_selfcheck.b_is_read_data != true || ob_selfcheck.f_is_read_data != true)
		{
			bll_imu_Resume_config_param(&game_bll_imu_param_t);
		}

		//¼ʱ
		sensor_t_count = t_count;
	}
	
	//fornt back mag data check shake
	if(t_count > (2000/100))
	{
		front_mag_norm = sqrt((double)(ob_selfcheck.f_data.mag[0]*ob_selfcheck.f_data.mag[0]) + (double)(ob_selfcheck.f_data.mag[1]*ob_selfcheck.f_data.mag[1]) + (double)(ob_selfcheck.f_data.mag[2]*ob_selfcheck.f_data.mag[2]));
		back_mag_norm = sqrt((double)(ob_selfcheck.b_data.mag[0]*ob_selfcheck.b_data.mag[0]) + (double)(ob_selfcheck.b_data.mag[1]*ob_selfcheck.b_data.mag[1]) + (double)(ob_selfcheck.b_data.mag[2]*ob_selfcheck.b_data.mag[2]));
		
		if(front_mag_norm >= SELFCHECK_SENSOR_MAG_SHAKE_TRIGGER_THRESHOLD)
		{
			//get max
			front_mag_norm_max = (front_mag_norm_max > front_mag_norm)?front_mag_norm_max:front_mag_norm;
			//get min
			front_mag_norm_min = (front_mag_norm_min > front_mag_norm)?front_mag_norm:front_mag_norm_min;
			
			front_mag_shake_trigger = true;
		}
		
		if(back_mag_norm >= SELFCHECK_SENSOR_MAG_SHAKE_TRIGGER_THRESHOLD)
		{
			//get max
			back_mag_norm_max = (back_mag_norm_max > back_mag_norm)?back_mag_norm_max:back_mag_norm;
			//get min
			back_mag_norm_min = (back_mag_norm_min > back_mag_norm)?back_mag_norm:back_mag_norm_min;
			
			back_mag_shake_trigger = true;
		}

		
		DEBUG_LOG("front_mag_norm_max:%d,%d,%d,%d,%d,%d,%d,%d,%d,%d\r\n",front_mag_norm,back_mag_norm,front_mag_norm_max,back_mag_norm_max, \
		                                                                front_mag_norm_min,back_mag_norm_min,front_mag_norm_max - front_mag_norm_min, \
		                                                                back_mag_norm_max - back_mag_norm_min,front_mag_shake_trigger,back_mag_shake_trigger);
		
		if(t_count >= (5000/100))
		{
			if(front_mag_norm_max - front_mag_norm_min >= SELFCHECK_SENSOR_MAG_SHAKE_THRESHOLD && front_mag_shake_trigger)
			{
				ob_selfcheck.selfcheck_result |= (1 << SELFCHECK_RESULT_ERR_FRONT_MAG_NO_DATA_OR_EXCP_DATA);
			}
			
			if(back_mag_norm_max - back_mag_norm_min >= SELFCHECK_SENSOR_MAG_SHAKE_THRESHOLD && back_mag_shake_trigger)
			{
				ob_selfcheck.selfcheck_result |= (1 << SELFCHECK_RESULT_ERR_BACK_NO_DATA_OR_EXCP_DATA);
			}
		}
	}

	//5鿴ܷȡ㲥֣СRSSIǷԼ
	if(t_count == (5000/100))
	{
		if(SELFCHECK_SCAN_DEVICE_RSSI_MIN_THRESHOLD >= ob_selfcheck.max_rssi)ob_selfcheck.selfcheck_result |= (1 << SELFCHECK_RESULT_ERR_RSSI);
	}

	//5ر100𶯵Լ̣߳ʼ40Լʾ̣߳holdonȫС
	if(t_count == (5000/100))
	{
		nrf_gpio_pin_write(PIN_MT_EN,0);
		Process_Stop(selfcheck_mt_process);
		Process_Start(0,"selfcheck_result_display_process",selfcheck_result_display_process);
		Process_SetHoldOn(selfcheck_result_display_process,1);
	}

	//ϱм䴫ļٶֵrollֵ
	L=0;
	int16_t rol  = (int16_t)(getRoll()*100);
	int16_t pitch = (int16_t)(getPitch()*100);
	int16_t yaw   = (int16_t)(getYaw()*100);
	buf[L++] = (uint8_t)(rol>>8);
	buf[L++] = (uint8_t)(rol>>0);
	buf[L++] = (uint8_t)(pitch>>8);
	buf[L++] = (uint8_t)(pitch>>0);
	buf[L++] = (uint8_t)(yaw>>8);
	buf[L++] = (uint8_t)(yaw>>0);
	buf[L++] = 0;
	buf[L++] = 0;
	buf[L++] = 0;
	buf[L++] = 0;
	buf[L++] = 0; 
	buf[L++] = 0;
	buf[L++] = 0;
	Mahony_send_ANO(0x01,buf,L);
	L=0;
	int32_t middle_acc_x = ob_selfcheck.m_data.acc[0];
	int32_t middle_acc_y = ob_selfcheck.m_data.acc[1];
	int16_t middle_acc_z = ob_selfcheck.m_data.acc[2];
	middle_acc_x *= 100;
	middle_acc_y *= 100;
	middle_acc_z *= 10;
	buf[L++] = (uint8_t)(middle_acc_x>>24);
	buf[L++] = (uint8_t)(middle_acc_x>>16);
	buf[L++] = (uint8_t)(middle_acc_x>>8);
	buf[L++] = (uint8_t)(middle_acc_x>>0);
	buf[L++] = (uint8_t)(middle_acc_y>>24);
	buf[L++] = (uint8_t)(middle_acc_y>>16);
	buf[L++] = (uint8_t)(middle_acc_y>>8);
	buf[L++] = (uint8_t)(middle_acc_y>>0);
	buf[L++] = (uint8_t)(middle_acc_z>>8);
	buf[L++] = (uint8_t)(middle_acc_z>>0);
	Mahony_send_ANO(0x07,buf,L);

	//ϱǰŴļٶݡݡشżƿƽ
	//ϱŴĵشżƿƽ
	//ϱɨ赽㲥rssiֵ
	if(TIME_GetTicks()-last_tim >= 1000)
	{
		last_tim = TIME_GetTicks();
		
		ob_selfcheck.max_rssi = -120;
	}
	L=0;
	buf[L++] = (uint8_t)(ob_selfcheck.f_data.acc[0]>>8);
	buf[L++] = (uint8_t)(ob_selfcheck.f_data.acc[0]>>0);
	buf[L++] = (uint8_t)(ob_selfcheck.f_data.acc[1]>>8);
	buf[L++] = (uint8_t)(ob_selfcheck.f_data.acc[1]>>0);
	buf[L++] = (uint8_t)(ob_selfcheck.f_data.acc[2]>>8);
	buf[L++] = (uint8_t)(ob_selfcheck.f_data.acc[2]>>0);
	buf[L++] = (uint8_t)(ob_selfcheck.f_data.gry[0]>>8);
	buf[L++] = (uint8_t)(ob_selfcheck.f_data.gry[0]>>0);
	buf[L++] = (uint8_t)(ob_selfcheck.f_data.gry[1]>>8);
	buf[L++] = (uint8_t)(ob_selfcheck.f_data.gry[1]>>0);
	buf[L++] = (uint8_t)(ob_selfcheck.f_data.gry[2]>>8);
	buf[L++] = (uint8_t)(ob_selfcheck.f_data.gry[2]>>0);
	front_mag_norm = sqrt((double)(ob_selfcheck.f_data.mag[0]*ob_selfcheck.f_data.mag[0]) + (double)(ob_selfcheck.f_data.mag[1]*ob_selfcheck.f_data.mag[1]) + (double)(ob_selfcheck.f_data.mag[2]*ob_selfcheck.f_data.mag[2]));
	back_mag_norm = sqrt((double)(ob_selfcheck.b_data.mag[0]*ob_selfcheck.b_data.mag[0]) + (double)(ob_selfcheck.b_data.mag[1]*ob_selfcheck.b_data.mag[1]) + (double)(ob_selfcheck.b_data.mag[2]*ob_selfcheck.b_data.mag[2]));
	buf[L++] = (uint8_t)(front_mag_norm>>8);
	buf[L++] = (uint8_t)(front_mag_norm>>0);
	buf[L++] = (uint8_t)(back_mag_norm>>8);
	buf[L++] = (uint8_t)(back_mag_norm>>0);
	buf[L++] = (uint8_t)(ob_selfcheck.max_rssi>> 8);
	buf[L++] = (uint8_t)(ob_selfcheck.max_rssi>>0);
	Mahony_send_ANO(0x02,buf,L);
	
	fml_adc_get_value(PIN_CHARGING_CHANNEL,&adc_value);
	if(adc_value >= 2000)
	{
		if(continue_trigger <= 50)continue_trigger++;
	}
	else
	{
		if(continue_trigger != 0)continue_trigger--;
	}
					
	//30
	if(t_count >= (30000/100) && !slave_isconnect() && continue_trigger==0)
	{
    memset(mac_buf, 0, sizeof(mac_buf));
    sprintf(mac_buf, "%02X%02X%02X%02X%02X%02X", mFlash.macHost[0], mFlash.macHost[1], mFlash.macHost[2], mFlash.mClient.macAddr[3], mFlash.mClient.macAddr[4], mFlash.mClient.macAddr[5]);
		ST_scan_stop();
    host_set_scan_name(mac_buf, strlen(mac_buf));
		//ݵflash
		if(Flash_SaveStep() != ZONE_OP_SUCCESS)Except_TxError(EXCEPT_Power,"save step fail");
		extern battercb_t battery_record;
//		extern void printbatter_cb(battercb_t *c,unsigned char *buff);
		memcpy(&mFlash.mbattercb_t,&battery_record,sizeof(battercb_t));
		mBackup.RestartCnt =0;
		if(Flash_SaveBackup() != ZONE_OP_SUCCESS)Except_TxError(EXCEPT_Power,"pwr save backup fail");
		if(Flash_SaveInfomation() != ZONE_OP_SUCCESS)Except_TxError(EXCEPT_Power,"pwr save information fail");
		NVIC_SystemReset();
	}
}

static void adc_callback(uint32_t sample_point, Fml_Adc_All_Channel_Adc_Value_t all_adc_value)
{
	//
}

/*API ------------------------------------------------------------------------------------------------------------------------------------*/
/**
	@brief 		Լ촥ص
	@param 		order							-	[in] صָ
	@return 	
*/
void selfcheck_trigger_callback(char order)
{
	int 							error;
	uint32_t					last_tim = TIME_GetTicks();
	
	//ι
	feed_watchdog();
	
	//ʼṹ壬ԼΪɹ
	memset(&ob_selfcheck,0,sizeof(ob_selfcheck));
	ob_selfcheck.max_rssi 											= -120;
	ob_selfcheck.selfcheck_result 							= SELFCHECK_RESULT_SUCCESS;
	ob_selfcheck.selfcheck_result_led_color			= COLOR_BLACK;
	ob_selfcheck.f_is_read_data									= false;
	ob_selfcheck.b_is_read_data									= false;
	ob_selfcheck.selfcheck_is_led_display       = false;
	
	//ر̣߳ʼ0Լ̣߳holdonȫС
	Process_All_Stop();
	Process_Start(100,"selfcheck_process",selfcheck_process);	
	Process_SetHoldOn(selfcheck_process,1);

	//رɨ裬豸ɨصɨ
	while(TIME_GetTicks() - last_tim <= 1000)
	{
		host_disconnect();
		if(host_isconnect() == 0)break;
	}
	ST_scan_stop();
	host_set_scan_name((char *)"***********",sizeof("***********"));
	advdata_report_Evt_Regist(scan_report_cb);
	error = ST_scan_start();
	if(error != APP_SUCCESS)ob_selfcheck.selfcheck_result |= (1 << SELFCHECK_RESULT_ERR_RSSI);

	//LEDԴΪSOD1
	nrf_gpio_cfg(
	PIN_LED_ENABLE,
	NRF_GPIO_PIN_DIR_OUTPUT,
	NRF_GPIO_PIN_INPUT_DISCONNECT,
	NRF_GPIO_PIN_NOPULL,
	NRF_GPIO_PIN_S0D1,
	NRF_GPIO_PIN_NOSENSE);

	//𶯵ΪSOS1
	nrf_gpio_cfg_output(PIN_MT_EN);
	nrf_gpio_pin_write(PIN_MT_EN,0);	

	//³ʼADCͨ
	
	ADC_SetPinChannel(PIN_ADC_CHARGMEASURE, PIN_ADC_CHARGMEASURE_CHANNEL,NRF_GPIO_PIN_NOPULL);
	ADC_SetPinChannel(PIN_ADC_BAT_IN, PIN_ADC_BAT_CHANNEL,NRF_GPIO_PIN_NOPULL);
	ADC_SetPinChannel(PIN_CHARGING, PIN_CHARGING_CHANNEL,NRF_GPIO_PIN_NOPULL);
	fml_adc_sample_update_notify_register(adc_callback);

	//ǰŴм䴫Ŵ
	bll_imu_Init();
	bll_imu_register_data_notify_callback(BLL_IMU_DATA_NOTIFY_CB_PRIORITY_1, fb_data_notify_cb);
	bll_imu_Resume_config_param(&game_bll_imu_param_t);
	error = drv_qma_Init();
	if(error == -1)ob_selfcheck.selfcheck_result |= (1 << SELFCHECK_RESULT_ERR_MIDDLE_SENSOR_CONFIG);
	nrf_delay_ms(20);
	error = drv_qma_set_acc_odr(QMA_ACC_ODR_104HZ);
	if(error == -1)ob_selfcheck.selfcheck_result |= (1 << SELFCHECK_RESULT_ERR_MIDDLE_SENSOR_CONFIG);
	
	//ʼ500ms𶯵Լ̣߳𶯵500msȻ500msڣ
	Process_Start(500,"selfcheck_mt_process",selfcheck_mt_process);	
	Process_SetHoldOn(selfcheck_mt_process,1);

	//ʼrollֵ㷨
	Mahony_Init(10);
	
	//ι
	feed_watchdog();	
	
}















